program Autocompress;
{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: Autocompress.dpr, released on 24 July 2005.

The Initial Developer of the Original Code is Alexey Nikitayev [nikitayev@rsu.ru]
All Rights Reserved.

Contributor(s):

Last Modified: 24 July 2005

You may retrieve the latest version of this file at the Project Opencomputer's home page,
located at http://opencomputer.narod.ru/

Known Issues:
  Compressing windows's home folder may coses troubles

Used components:
  JVCL - www1:http://jvcl.sf.net/daily 
         www2:http://homepages.borland.com/jedi/jvcl/ 
  description: the best opensource components. 
         .. RxLib, RaLib  ..  .
  Delphi4-7

-----------------------------------------------------------------------------}

{$APPTYPE CONSOLE}

uses
  Windows,SysUtils,StrUtils,Classes,
  zlib, ZLibConst, JclFileUtils,
  JclWin32,JclNTFS;

function CheckDriveForCompressing (const FileName:string):boolean;
var
  MaximumComponentLength,FileSystemFlags:DWORD;
  DriveStr:string;
begin
  DriveStr:=ExtractFileDrive(FileName);
  GetVolumeInformation(PChar(DriveStr),nil,0,nil,
    MaximumComponentLength,FileSystemFlags,nil,0);
  result:=(FileSystemFlags and FS_FILE_COMPRESSION=FS_FILE_COMPRESSION	);
end;

function CCheck(code: Integer): Integer;
begin
  Result := code;
  if code < 0 then
    raise ECompressionError.Create(sError); //!!
end;

procedure CompressBuf2(const InBuf: Pointer; InBytes: Integer;
                      out OutBytes: Integer);
var
  strm: TZStreamRec;
  P: Pointer;
  OutBuf: Pointer;
begin
  FillChar(strm, sizeof(strm), 0);
  strm.zalloc := zlibAllocMem;
  strm.zfree := zlibFreeMem;
  OutBytes := ((InBytes + (InBytes div 10) + 12) + 255) and not 255;
  GetMem(OutBuf, OutBytes);
  try
    strm.next_in := InBuf;
    strm.avail_in := InBytes;
    strm.next_out := OutBuf;
    strm.avail_out := OutBytes;
    CCheck(deflateInit_(strm, 1, zlib_version, sizeof(strm)));
    try
      while CCheck(deflate(strm, Z_FINISH)) <> Z_STREAM_END do
      begin
        P := OutBuf;
        Inc(OutBytes, 256);
        ReallocMem(OutBuf, OutBytes);
        strm.next_out := PChar(Integer(OutBuf) + (Integer(strm.next_out) - Integer(P)));
        strm.avail_out := 256;
      end;
    finally
      CCheck(deflateEnd(strm));
    end;
    OutBytes := strm.total_out;
  except
  end;
  FreeMem(OutBuf);
end;

function TryToCompress (const FileName:string;var SizeBefore, SizeAfter:Int64):double;
var
  FileSize:Int64;
  MS:TMemoryStream;
  CompressedSize:Integer;
begin
  result:=1.000;
  SizeBefore:=0;
  SizeAfter:=0;
  MS:=TMemoryStream.Create;
  try
    FileSize:=GetSizeOfFile(FileName);
    if (FileSize>4096) and (FileSize<10*1024*1024) then
    begin
      MS.LoadFromFile(FileName);
      if (MS.Size<>0) then
      begin
        SizeBefore:=MS.Size;
        CompressBuf2(MS.Memory,MS.Size,CompressedSize);
        SizeAfter:=CompressedSize;
        if (CompressedSize<>0) then
          result:=CompressedSize/MS.Size
        else
          result:=1.000;
      end;
    end;
  except
  end;
  FreeAndNil(MS);
end;

procedure SetPathCompression(Dir: string; const Mask: string;
  Recursive, ApplyCompress: Boolean; 
  var FilesCount:Integer; var SizeBefore, SizeAfter:Int64);
var
  FileName: string;
  SearchRec: TSearchRec;
  R: Integer;
  FileSizeBefore, FileSizeAfter:Int64;
  CompState:TFileCompressionState;
begin
    Dir := PathAddSeparator(Dir);
    if FindFirst(Dir + Mask, faAnyFile, SearchRec) = 0 then
    try
      repeat
        if (SearchRec.Name <> '.') and (SearchRec.Name <> '..') then
        begin
          FileName := Dir + SearchRec.Name;
          if (SearchRec.Attr and faDirectory) = 0 then
          begin
            CompState:=NtfsGetCompression(FileName);
            if (CompState=fcNoCompression) and 
               (TryToCompress(FileName, FileSizeBefore, FileSizeAfter)<0.8) then
            begin
              if (ApplyCompress)then
                NtfsSetCompression(FileName,COMPRESSION_FORMAT_DEFAULT);
              Inc(SizeBefore,FileSizeBefore);
              Inc(SizeAfter,FileSizeAfter);
              Inc(FilesCount);
            end;
          end
          else
            if Recursive then
              SetPathCompression(FileName, Mask, True,ApplyCompress,FilesCount,SizeBefore,SizeAfter);
        end;
        R := FindNext(SearchRec);
      until R <> 0;
    finally
      SysUtils.FindClose(SearchRec);
    end;
end;

procedure ShowHelp;
begin
  WriteLn('directory name as parameter expected:');
  WriteLn('sample: "autocompress.exe /R /A c:\"');
  WriteLn('supported parameters:');
  WriteLn('/? - Help');
  WriteLn('/R - recursive');
  WriteLn('/A - apply compression');
  WriteLn('ATTENTION!');
  WriteLn('  Compressing windows''s home folder may coses troubles');
end;

var
  Directory:string;
  SizeBefore, SizeAfter:Int64;
  FilesCount:Integer;
  i,pf:Integer;
  IsRecursive:boolean;
  IsApplyCompression:boolean;
function CheckForParameter(const param:string):boolean;
begin
  result:=false;
  if (param='/?') then
  begin
    ShowHelp;
    result:=true;
  end else
  if (param='/R') then
  begin
    WriteLn('recursive processing ON');
    IsRecursive:=true;
    result:=true;
  end else
  if (param='/A') then
  begin
    WriteLn('apply compression ON');
    IsApplyCompression:=true;
    result:=true;
  end;
end;
begin
  IsRecursive:=false;
  IsApplyCompression:=false;
  WriteLn('Autocompress [Version 1.0]'#13#10+
          '(C) Alexey V. Nikitayev, 2005 july.'#13#10+
          'license: free "AS IS" not for commercial use'#13#10+
          'latest version: http://opencomputer.narod.ru/');
  WriteLn;
  if (ParamCount=0) then
  begin
    ShowHelp;
    exit;
  end;
  
  //     
  pf:=1;
  while CheckForParameter(ParamStr(pf)) do Inc(pf);

  //    
  Directory:=ParamStr(pf);
  for i:=pf+1 to ParamCount do
    Directory:=Directory+' '+ParamStr(i);

  //  ,      
  WriteLn(Format('processing <%s>',[Directory]));
  if (not DirectoryExists(Directory)) then
  begin
    WriteLn(Format('directory <%s>',[Directory]));
    WriteLn('is not exists!');
    exit;
  end;

  //   ,         
  if not CheckDriveForCompressing(Directory) then
  begin
    WriteLn(Format('on drive <%s> compression is not supported',[ExtractFileDrive(Directory)]));
    exit;
  end;

  //   
  SizeBefore:=0;
  SizeAfter:=0;
  FilesCount:=0;
  SetPathCompression(Directory,'*.*',IsRecursive,IsApplyCompression,FilesCount,SizeBefore,SizeAfter);
  WriteLn(Format(' Files count:  %d',[FilesCount]));
  WriteLn(Format(' Size before:  %d',[SizeBefore]));
  WriteLn(Format('  Size after:  %d',[SizeAfter]));
  if (SizeBefore<>0) then
    WriteLn(Format('aspect ratio:  %f',[SizeAfter/SizeBefore]));
end.
